@@ -35,7 +35,18 @@ allowed_cursor = ['alias', 'cell', 'grab', 'move', 'crosshair', 'context-menu',
 
 
 def basemap_to_tiles(basemap, day='yesterday', **kwargs):
-    # Format the URL with modisdate
+    """Turn a basemap into a TileLayer object.
+
+    Parameters
+    ----------
+    basemap : dict
+        Basemap description coming from ipyleaflet.basemaps.
+    day: string
+        If relevant for the chosen basemap, you can specify the day for
+        the tiles in the "%Y-%m-%d" format. Defaults to "yesterday".
+    kwargs: key-word arguments
+        Extra key-word arguments to pass to the TileLayer constructor.
+    """
     from datetime import date, timedelta
 
     if day == 'yesterday':
@@ -76,6 +87,18 @@ class InteractMixin(object):
 
 
 class Layer(Widget, InteractMixin):
+    """Abstract Layer class.
+
+    Base class for all layers in ipyleaflet.
+
+    Attributes
+    ----------
+    name : string
+        Custom name for the layer, which will be used by the LayersControl.
+    popup: object
+        Interactive widget that will be shown in a Popup when clicking on the layer.
+    """
+
     _view_name = Unicode('LeafletLayerView').tag(sync=True)
     _model_name = Unicode('LeafletLayerModel').tag(sync=True)
     _view_module = Unicode('jupyter-leaflet').tag(sync=True)
@@ -126,43 +149,155 @@ class Layer(Widget, InteractMixin):
             self._mouseout_callbacks(**content)
 
     def on_click(self, callback, remove=False):
+        """Add a click event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called on click event.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._click_callbacks.register_callback(callback, remove=remove)
 
     def on_dblclick(self, callback, remove=False):
+        """Add a double-click event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called on double-click event.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._dblclick_callbacks.register_callback(callback, remove=remove)
 
     def on_mousedown(self, callback, remove=False):
+        """Add a mouse-down event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called on mouse-down event.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._mousedown_callbacks.register_callback(callback, remove=remove)
 
     def on_mouseup(self, callback, remove=False):
+        """Add a mouse-up event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called on mouse-up event.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._mouseup_callbacks.register_callback(callback, remove=remove)
 
     def on_mouseover(self, callback, remove=False):
+        """Add a mouse-over event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called on mouse-over event.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._mouseover_callbacks.register_callback(callback, remove=remove)
 
     def on_mouseout(self, callback, remove=False):
+        """Add a mouse-out event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called on mouse-out event.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._mouseout_callbacks.register_callback(callback, remove=remove)
 
 
 class UILayer(Layer):
+    """Abstract UILayer class."""
+
     _view_name = Unicode('LeafletUILayerView').tag(sync=True)
     _model_name = Unicode('LeafletUILayerModel').tag(sync=True)
 
 
 class Icon(UILayer):
+    """Icon class.
+
+    Custom icon used for markers.
+
+    Attributes
+    ----------
+    icon_url : string
+        The url to the image used for the icon.
+    shadow_url: string, default None
+        The url to the image used for the icon shadow.
+    icon_size: tuple, default None
+        The size of the icon, in pixels.
+    shadow_size: tuple, default None
+        The size of the icon shadow, in pixels.
+    icon_anchor: tuple, default None
+        The coordinates of the "tip" of the icon (relative to its top left corner).
+        The icon will be aligned so that this point is at the marker's geographical
+        location. Centered by default if icon_size is specified.
+    shadow_anchor: tuple, default None
+        The coordinates of the "tip" of the shadow (relative to its top left corner).
+        The same as icon_anchor if None.
+    popup_anchor: tuple, default None
+        The coordinates of the point from which popups will "open", relative to the
+        icon anchor.
+    """
+
     _view_name = Unicode('LeafletIconView').tag(sync=True)
     _model_name = Unicode('LeafletIconModel').tag(sync=True)
 
     icon_url = Unicode('').tag(sync=True, o=True)
     shadow_url = Unicode(None, allow_none=True).tag(sync=True, o=True)
-    icon_size = Tuple((10, 10), allow_none=True).tag(sync=True, o=True)
-    shadow_size = Tuple((10, 10), allow_none=True).tag(sync=True, o=True)
-    icon_anchor = Tuple((0, 0), allow_none=True).tag(sync=True, o=True)
-    shadow_anchor = Tuple((0, 0), allow_none=True).tag(sync=True, o=True)
-    popup_anchor = Tuple((0, 0), allow_none=True).tag(sync=True, o=True)
+    icon_size = List(default_value=None, allow_none=True).tag(sync=True, o=True)
+    shadow_size = List(default_value=None, allow_none=True).tag(sync=True, o=True)
+    icon_anchor = List(default_value=None, allow_none=True).tag(sync=True, o=True)
+    shadow_anchor = List(default_value=None, allow_none=True).tag(sync=True, o=True)
+    popup_anchor = List([0, 0], allow_none=True).tag(sync=True, o=True)
+
+    @validate('icon_size', 'shadow_size', 'icon_anchor', 'shadow_anchor', 'popup_anchor')
+    def _validate_attr(self, proposal):
+        value = proposal['value']
+
+        # Workaround Traitlets which does not respect the None default value
+        if value is None or len(value) == 0:
+            return None
+
+        if len(value) != 2:
+            raise TraitError('The value should be of size 2, but {} was given'.format(value))
+
+        return value
 
 
 class AwesomeIcon(UILayer):
+    """AwesomeIcon class.
+
+    FontAwesome icon used for markers.
+
+    Attributes
+    ----------
+    name : string, default "home"
+        The name of the FontAwesome icon to use.
+        See https://fontawesome.com/v4.7.0/icons for available icons.
+    marker_color: string, default "blue"
+        Color used for the icon background.
+    icon_color: string, default "white"
+        CSS color used for the FontAwesome icon.
+    spin: boolean, default False
+        Whether the icon is spinning or not.
+    """
+
     _view_name = Unicode('LeafletAwesomeIconView').tag(sync=True)
     _model_name = Unicode('LeafletAwesomeIconModel').tag(sync=True)
 
@@ -178,6 +313,35 @@ class AwesomeIcon(UILayer):
 
 
 class Marker(UILayer):
+    """Marker class.
+
+    Clickable/Draggable marker on the map.
+
+    Attributes
+    ----------
+    location: tuple, default (0, 0)
+        The tuple containing the latitude/longitude of the marker.
+    opacity: float, default 1.
+        Opacity of the marker between 0. (fully transparent) and 1. (fully opaque).
+    visible: boolean, default True
+        Whether the marker is visible or not.
+    icon: object, default None
+        Icon used for the marker, it can be an Icon or an AwesomeIcon instance.
+        By default it will use a blue icon.
+    draggable: boolean, default True
+        Whether the marker is draggable with the mouse or not.
+    keyboard: boolean, default True
+        Whether the marker can be tabbed to with a keyboard and clicked by pressing enter.
+    title: string, default ''
+        Text for the browser tooltip that appear on marker hover (no tooltip by default).
+    alt: string, default ''
+        Text for the alt attribute of the icon image (useful for accessibility).
+    rotation_angle: float, default 0.
+        The rotation angle of the marker in degrees.
+    rotation_origin: string, default ''
+        The rotation origin of the marker.
+    """
+
     _view_name = Unicode('LeafletMarkerView').tag(sync=True)
     _model_name = Unicode('LeafletMarkerModel').tag(sync=True)
 
@@ -209,10 +373,50 @@ class Marker(UILayer):
             self._move_callbacks(**content)
 
     def on_move(self, callback, remove=False):
+        """Add a move event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called on move event.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._move_callbacks.register_callback(callback, remove=remove)
 
 
 class Popup(UILayer):
+    """Popup class.
+
+    Popup element that can be placed on the map.
+
+    Attributes
+    ----------
+    location: tuple, default (0, 0)
+        The tuple containing the latitude/longitude of the popup.
+    child: object, default None
+        Child widget that the Popup will contain.
+    min_width: int, default 50
+        Minimum width of the Popup.
+    max_width: int, default 300
+        Maximum width of the Popup.
+    max_height: int, default None
+        Maximum height of the Popup.
+    auto_pan: boolean, default True
+        Set it to False if you don’t want the map to do panning
+        animation to fit the opened popup.
+    keep_in_view: boolean, default False
+        Set it to True if you want to prevent users from panning
+        the popup off of the screen while it is open.
+    close_button: boolean, default True
+        Whether to show a close button or not.
+    auto_close: boolean, default True
+        Whether to automatically close the popup when interacting
+        with another element of the map or not.
+    close_on_escape_key: boolean, default True
+        Whether to close the popup when clicking the escape key or not.
+    """
+
     _view_name = Unicode('LeafletPopupView').tag(sync=True)
     _model_name = Unicode('LeafletPopupModel').tag(sync=True)
 
@@ -237,6 +441,16 @@ class Popup(UILayer):
 
 
 class RasterLayer(Layer):
+    """Abstract RasterLayer class.
+
+    Attributes
+    ----------
+    opacity: float, default 1.
+        Opacity of the layer between 0. (fully transparent) and 1. (fully opaque).
+    visible: boolean, default True
+        Whether the layer is visible or not.
+    """
+
     _view_name = Unicode('LeafletRasterLayerView').tag(sync=True)
     _model_name = Unicode('LeafletRasterLayerModel').tag(sync=True)
 
@@ -245,6 +459,32 @@ class RasterLayer(Layer):
 
 
 class TileLayer(RasterLayer):
+    """TileLayer class.
+
+    Tile service layer.
+
+    Attributes
+    ----------
+    url: string, default "https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
+        Url to the tiles service.
+    min_zoom: int, default 0
+        Minimum zoom for this tile service.
+    max_zoom: int, default 18
+        Maximum zoom for this tile service.
+    tile_size int, default 256
+        Tile sizes for this tile service.
+    attribution string, default "Map data (c) <a href="https://openstreetmap.org">OpenStreetMap</a> contributors"
+        Tiles service attribution.
+    no_wrap boolean, default False
+        Whether the layer is wrapped around the antimeridian.
+    tms: boolean, default False
+        If true, inverses Y axis numbering for tiles (turn this on for TMS services).
+    show_loading: boolean, default False
+        Whether to show a spinner when tiles are loading.
+    loading: boolean, default False
+        Whether the tiles are currently loading.
+    """
+
     _view_name = Unicode('LeafletTileLayerView').tag(sync=True)
     _model_name = Unicode('LeafletTileLayerModel').tag(sync=True)
 
@@ -274,13 +514,39 @@ class TileLayer(RasterLayer):
             self._load_callbacks(**content)
 
     def on_load(self, callback, remove=False):
+        """Add a load event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called when the tiles have finished loading.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._load_callbacks.register_callback(callback, remove=remove)
 
     def redraw(self):
+        """Force redrawing the tiles.
+
+        This is especially useful when you are sure the server updated the tiles and you
+        need to refresh the layer.
+        """
         self.send({'msg': 'redraw'})
 
 
 class LocalTileLayer(TileLayer):
+    """LocalTileLayer class.
+
+    Custom tile layer using local tile files.
+
+    Attributes
+    ----------
+    path: string, default ""
+        Path to your local tiles. In the classic Jupyter Notebook, the path is relative to
+        the Notebook you are working on. In JupyterLab, the path is relative to the server
+        (where you started JupyterLab) and you need to prefix the path with “files/”.
+    """
+
     _view_name = Unicode('LeafletLocalTileLayerView').tag(sync=True)
     _model_name = Unicode('LeafletLocalTileLayerModel').tag(sync=True)
 
@@ -288,13 +554,25 @@ class LocalTileLayer(TileLayer):
 
 
 class WMSLayer(TileLayer):
+    """WMSLayer class.
+
+    Attributes
+    ----------
+    layers: string, default ""
+        Comma-separated list of WMS layers to show.
+    styles: string, default ""
+        Comma-separated list of WMS styles
+    format: string, default "image/jpeg"
+        WMS image format (use `'image/png'` for layers with transparency).
+    transparent: boolean, default False
+        If true, the WMS service will return images with transparency.
+    crs: dict, default ipyleaflet.projections.EPSG3857
+        Projection used for this WMS service.
+    """
+
     _view_name = Unicode('LeafletWMSLayerView').tag(sync=True)
     _model_name = Unicode('LeafletWMSLayerModel').tag(sync=True)
 
-    service = Unicode('WMS').tag(sync=True)
-    request = Unicode('GetMap').tag(sync=True)
-    version = Unicode('1.1.1').tag(sync=True)
-
     # Options
     layers = Unicode().tag(sync=True, o=True)
     styles = Unicode().tag(sync=True, o=True)
@@ -305,6 +583,20 @@ class WMSLayer(TileLayer):
 
 
 class ImageOverlay(RasterLayer):
+    """ImageOverlay class.
+
+    Image layer from a local or remote image file.
+
+    Attributes
+    ----------
+    url: string, default ""
+        Url to the local or remote image file.
+    bounds: list, default [0., 0]
+        SW and NE corners of the image.
+    attribution: string, default ""
+        Image attribution.
+    """
+
     _view_name = Unicode('LeafletImageOverlayView').tag(sync=True)
     _model_name = Unicode('LeafletImageOverlayModel').tag(sync=True)
 
@@ -316,6 +608,20 @@ class ImageOverlay(RasterLayer):
 
 
 class VideoOverlay(RasterLayer):
+    """VideoOverlay class.
+
+    Video layer from a local or remote video file.
+
+    Attributes
+    ----------
+    url: string, default ""
+        Url to the local or remote video file.
+    bounds: list, default [0., 0]
+        SW and NE corners of the video.
+    attribution: string, default ""
+        Video attribution.
+    """
+
     _view_name = Unicode('LeafletVideoOverlayView').tag(sync=True)
     _model_name = Unicode('LeafletVideoOverlayModel').tag(sync=True)
 
@@ -327,6 +633,22 @@ class VideoOverlay(RasterLayer):
 
 
 class Heatmap(RasterLayer):
+    """Heatmap class.
+
+    Heatmap layer.
+
+    Attributes
+    ----------
+    locations: list, default []
+        List of data points locations for generating the heatmap.
+    radius: float, default 25.
+        Radius of the data points.
+    blur: float, default 15.
+        Blurring intensity.
+    gradient: dict, default {0.4: 'blue', 0.6: 'cyan', 0.7: 'lime', 0.8: 'yellow', 1.0: 'red'}
+        Colors used for the color-mapping from low to high heatmap intensity.
+    """
+
     _view_name = Unicode('LeafletHeatmapView').tag(sync=True)
     _model_name = Unicode('LeafletHeatmapModel').tag(sync=True)
 
@@ -342,6 +664,20 @@ class Heatmap(RasterLayer):
 
 
 class VectorTileLayer(Layer):
+    """VectorTileLayer class.
+
+    Vector tile layer.
+
+    Attributes
+    ----------
+    url: string, default ""
+        Url to the vector tile service.
+    attribution: string, default ""
+        Vector tile service attribution.
+    vector_tile_layer_styles: dict, default {}
+        CSS Styles to apply to the vector data.
+    """
+
     _view_name = Unicode('LeafletVectorTileLayerView').tag(sync=True)
     _model_name = Unicode('LeafletVectorTileLayerModel').tag(sync=True)
 
@@ -351,15 +687,49 @@ class VectorTileLayer(Layer):
     vector_tile_layer_styles = Dict().tag(sync=True, o=True)
 
     def redraw(self):
+        """Force redrawing the tiles.
+
+        This is especially useful when you are sure the server updated the tiles and you
+        need to refresh the layer.
+        """
         self.send({'msg': 'redraw'})
 
 
 class VectorLayer(Layer):
+    """VectorLayer abstract class."""
+
     _view_name = Unicode('LeafletVectorLayerView').tag(sync=True)
     _model_name = Unicode('LeafletVectorLayerModel').tag(sync=True)
 
 
 class Path(VectorLayer):
+    """Path abstract class.
+
+    Path layer.
+
+    Attributes
+    ----------
+    stroke: boolean, default True
+        Whether to draw a stroke.
+    color: CSS color, default '#0033FF'
+        CSS color.
+    weight: int, default 5
+        Weight of the stroke.
+    fill: boolean, default True
+        Whether to fill the path with a flat color.
+    fill_color: CSS color, default None
+        Color used for filling the path shape. If None, the color attribute
+        value is used.
+    fill_opacity: float, default 0.2
+        Opacity used for filling the path shape.
+    line_cap: string, default "round"
+        A string that defines shape to be used at the end of the stroke.
+        Possible values are 'round', 'butt' or 'square'.
+    line_join: string, default "round"
+        A string that defines shape to be used at the corners of the stroke.
+        Possible values are 'arcs', 'bevel', 'miter', 'miter-clip' or 'round'.
+    """
+
     _view_name = Unicode('LeafletPathView').tag(sync=True)
     _model_name = Unicode('LeafletPathModel').tag(sync=True)
 
@@ -371,14 +741,43 @@ class Path(VectorLayer):
     fill_color = Color(None, allow_none=True).tag(sync=True, o=True)
     fill_opacity = Float(0.2).tag(sync=True, o=True)
     dash_array = Unicode(allow_none=True, default_value=None).tag(sync=True, o=True)
-    line_cap = Unicode('round').tag(sync=True, o=True)
-    line_join = Unicode('round').tag(sync=True, o=True)
+    line_cap = Enum(values=['round', 'butt', 'square'], default_value='round').tag(sync=True, o=True)
+    line_join = Enum(values=['arcs', 'bevel', 'miter', 'miter-clip', 'round'], default_value='round').tag(sync=True, o=True)
     pointer_events = Unicode('').tag(sync=True, o=True)
     class_name = Unicode('').tag(sync=True, o=True)
     opacity = Float(1.0, min=0.0, max=1.0).tag(sync=True, o=True)
 
 
 class AntPath(VectorLayer):
+    """AntPath class.
+
+    AntPath layer.
+
+    Attributes
+    ----------
+    locations: list, default []
+        Locations through which the ant-path is going.
+    use: string, default 'polyline'
+        Type of shape to use for the ant-path. Possible values are 'polyline', 'polygon',
+        'rectangle' and 'circle'.
+    delay: int, default 400
+        Add a delay to the animation flux.
+    weight: int, default 5
+        Weight of the ant-path.
+    dash_array: list, default [10, 20]
+        The sizes of the animated dashes.
+    color: CSS color, default '#0000FF'
+        The color of the primary dashes.
+    pulse_color: CSS color, default '#FFFFFF'
+        The color of the secondary dashes.
+    paused: boolean, default False
+        Whether the animation is running or not.
+    reverse: boolean, default False
+        Whether the animation is going backwards or not.
+    hardware_accelerated: boolean, default False
+        Whether the ant-path uses hardware acceleration.
+    """
+
     _view_name = Unicode('LeafletAntPathView').tag(sync=True)
     _model_name = Unicode('LeafletAntPathModel').tag(sync=True)
 
@@ -398,6 +797,26 @@ class AntPath(VectorLayer):
 
 
 class Polyline(Path):
+    """Polyline abstract class.
+
+    Attributes
+    ----------
+    locations: list, default []
+        Locations defining the shape.
+    scaling: boolean, default True
+        Whether you can edit the scale of the shape or not.
+    rotation: boolean, default True
+        Whether you can rotate the shape or not.
+    uniform_scaling: boolean, default False
+        Whether to keep the size ratio when changing the shape scale.
+    smooth_factor: float, default 1.
+        Smoothing intensity.
+    transform: boolean, default False
+        Whether the shape is editable or not.
+    draggable: boolean, default False
+        Whether you can drag the shape on the map or not.
+    """
+
     _view_name = Unicode('LeafletPolylineView').tag(sync=True)
     _model_name = Unicode('LeafletPolylineModel').tag(sync=True)
 
@@ -414,11 +833,26 @@ class Polyline(Path):
 
 
 class Polygon(Polyline):
+    """Polygon class.
+
+    Polygon layer.
+    """
+
     _view_name = Unicode('LeafletPolygonView').tag(sync=True)
     _model_name = Unicode('LeafletPolygonModel').tag(sync=True)
 
 
 class Rectangle(Polygon):
+    """Rectangle class.
+
+    Rectangle layer.
+
+    Attributes
+    ----------
+    bounds: list, default []
+        List of SW and NE location tuples. e.g. [(50, 75), (75, 120)].
+    """
+
     _view_name = Unicode('LeafletRectangleView').tag(sync=True)
     _model_name = Unicode('LeafletRectangleModel').tag(sync=True)
 
@@ -426,6 +860,18 @@ class Rectangle(Polygon):
 
 
 class CircleMarker(Path):
+    """CircleMarker class.
+
+    CircleMarker layer.
+
+    Attributes
+    ----------
+    location: list, default [0, 0]
+        Location of the marker (lat, long).
+    radius: int, default 10
+        Radius of the circle marker in pixels.
+    """
+
     _view_name = Unicode('LeafletCircleMarkerView').tag(sync=True)
     _model_name = Unicode('LeafletCircleMarkerModel').tag(sync=True)
 
@@ -436,6 +882,16 @@ class CircleMarker(Path):
 
 
 class Circle(CircleMarker):
+    """Circle class.
+
+    Circle layer.
+
+    Attributes
+    ----------
+    radius: int, default 1000
+        Radius of the circle marker in meters.
+    """
+
     _view_name = Unicode('LeafletCircleView').tag(sync=True)
     _model_name = Unicode('LeafletCircleModel').tag(sync=True)
 
@@ -444,6 +900,16 @@ class Circle(CircleMarker):
 
 
 class MarkerCluster(Layer):
+    """MarkerCluster class.
+
+    A cluster of markers that you can put on the map like other layers.
+
+    Attributes
+    ----------
+    markers: list, default []
+        List of markers to include in the cluster.
+    """
+
     _view_name = Unicode('LeafletMarkerClusterView').tag(sync=True)
     _model_name = Unicode('LeafletMarkerClusterModel').tag(sync=True)
 
@@ -451,6 +917,16 @@ class MarkerCluster(Layer):
 
 
 class LayerGroup(Layer):
+    """LayerGroup class.
+
+    A group of layers that you can put on the map like other layers.
+
+    Attributes
+    ----------
+    layers: list, default []
+        List of layers to include in the group.
+    """
+
     _view_name = Unicode('LeafletLayerGroupView').tag(sync=True)
     _model_name = Unicode('LeafletLayerGroupModel').tag(sync=True)
 
@@ -465,40 +941,85 @@ class LayerGroup(Layer):
         Makes sure only one instance of any given layer can exist in the
         layers list.
         '''
-        self._layer_ids = [l.model_id for l in proposal.value]
+        self._layer_ids = [layer.model_id for layer in proposal.value]
         if len(set(self._layer_ids)) != len(self._layer_ids):
             raise LayerException('duplicate layer detected, only use each layer once')
         return proposal.value
 
     def add_layer(self, layer):
+        """Add a new layer to the group.
+
+        Parameters
+        ----------
+        layer: layer instance
+            The new layer to include in the group.
+        """
         if isinstance(layer, dict):
             layer = basemap_to_tiles(layer)
         if layer.model_id in self._layer_ids:
             raise LayerException('layer already in layergroup: %r' % layer)
-        self.layers = tuple([l for l in self.layers] + [layer])
+        self.layers = tuple([layer for layer in self.layers] + [layer])
 
-    def remove_layer(self, layer):
-        if layer.model_id not in self._layer_ids:
-            raise LayerException('layer not on in layergroup: %r' % layer)
-        self.layers = tuple([l for l in self.layers if l.model_id != layer.model_id])
+    def remove_layer(self, rm_layer):
+        """Remove a layer from the group.
+
+        Parameters
+        ----------
+        layer: layer instance
+            The layer to remove from the group.
+        """
+        if rm_layer.model_id not in self._layer_ids:
+            raise LayerException('layer not on in layergroup: %r' % rm_layer)
+        self.layers = tuple([layer for layer in self.layers if layer.model_id != rm_layer.model_id])
 
     def substitute_layer(self, old, new):
+        """Substitute a layer with another one in the group.
+
+        Parameters
+        ----------
+        old: layer instance
+            The layer to remove from the group.
+        new: layer instance
+            The new layer to include in the group.
+        """
         if isinstance(new, dict):
             new = basemap_to_tiles(new)
         if old.model_id not in self._layer_ids:
             raise LayerException('Could not substitute layer: layer not in layergroup.')
-        self.layers = tuple([new if l.model_id == old.model_id else l for l in self.layers])
+        self.layers = tuple([new if layer.model_id == old.model_id else layer for layer in self.layers])
 
     def clear_layers(self):
+        """Remove all layers from the group."""
         self.layers = ()
 
 
 class FeatureGroup(LayerGroup):
+    """FeatureGroup abstract class."""
+
     _view_name = Unicode('LeafletFeatureGroupView').tag(sync=True)
     _model_name = Unicode('LeafletFeatureGroupModel').tag(sync=True)
 
 
 class GeoJSON(FeatureGroup):
+    """GeoJSON class.
+
+    Layer created from a GeoJSON data structure.
+
+    Attributes
+    ----------
+    data: dict, default {}
+        The JSON data structure.
+    style: dict, default {}
+        Extra style to apply to the features.
+    hover_style: dict, default {}
+        Style that will be applied to a feature when the mouse is over this feature.
+    point_style: dict, default {}
+        Extra style to apply to the point features.
+    style_callback: callable, default None
+        Function that will be called for each feature, should take the feature as
+        input and return the feature style.
+    """
+
     _view_name = Unicode('LeafletGeoJSONView').tag(sync=True)
     _model_name = Unicode('LeafletGeoJSONModel').tag(sync=True)
 
@@ -545,6 +1066,16 @@ class GeoJSON(FeatureGroup):
 
         return self.data
 
+    @property
+    def __geo_interface__(self):
+        """
+        Return a dict whose structure aligns to the GeoJSON format
+        For more information about the ``__geo_interface__``, see
+        https://gist.github.com/sgillies/2217756
+        """
+
+        return self.data
+
     def _apply_style(self, feature, style_callback):
         if 'properties' not in feature:
             feature['properties'] = {}
@@ -559,29 +1090,52 @@ class GeoJSON(FeatureGroup):
 
     def __init__(self, **kwargs):
         super(GeoJSON, self).__init__(**kwargs)
-        self.on_msg(self._handle_m_msg)
         self.data = self._get_data()
 
-    def _handle_m_msg(self, _, content, buffers):
+    def _handle_mouse_events(self, _, content, buffers):
         if content.get('event', '') == 'click':
             self._click_callbacks(**content)
         if content.get('event', '') == 'mouseover':
             self._hover_callbacks(**content)
 
     def on_click(self, callback, remove=False):
-        '''
-        The click callback takes an unpacked set of keyword arguments.
-        '''
+        """Add a feature click event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called on click event on a feature, this function
+            should take the event and the feature as inputs.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._click_callbacks.register_callback(callback, remove=remove)
 
     def on_hover(self, callback, remove=False):
-        '''
-        The hover callback takes an unpacked set of keyword arguments.
-        '''
+        """Add a feature hover event listener.
+
+        Parameters
+        ----------
+        callback : callable
+            Callback function that will be called when the mouse is over a feature, this function
+            should take the event and the feature as inputs.
+        remove: boolean
+            Whether to remove this callback or not. Defaults to False.
+        """
         self._hover_callbacks.register_callback(callback, remove=remove)
 
 
 class GeoData(GeoJSON):
+    """GeoData class.
+
+    Layer created from a GeoPandas dataframe.
+
+    Attributes
+    ----------
+    geo_dataframe: geopandas.GeoDataFrame instance, default None
+        The GeoPandas dataframe to use.
+    """
+
     geo_dataframe = Instance('geopandas.GeoDataFrame')
 
     def __init__(self, **kwargs):
@@ -595,8 +1149,38 @@ class GeoData(GeoJSON):
     def _get_data(self):
         return json.loads(self.geo_dataframe.to_json())
 
+    @property
+    def __geo_interface__(self):
+        """
+        Return a dict whose structure aligns to the GeoJSON format
+        For more information about the ``__geo_interface__``, see
+        https://gist.github.com/sgillies/2217756
+        """
+
+        return self.geo_dataframe.__geo_interface__
+
 
 class Choropleth(GeoJSON):
+    """Choropleth class.
+
+    Layer showing a Choropleth effect on a GeoJSON structure.
+
+    Attributes
+    ----------
+    geo_data: dict, default None
+        The GeoJSON structure on which to apply the Choropleth effect.
+    choro_data: dict, default None
+        Data used for building the Choropleth effect.
+    value_min: float, default None
+        Minimum data value for the color mapping.
+    value_max: float, default None
+        Maximum data value for the color mapping.
+    colormap: branca.colormap.ColorMap instance, default None
+        The colormap used for the effect.
+    key_on: string, default "id"
+        The feature key to use for the colormap effect.
+    """
+
     geo_data = Dict()
     choro_data = Dict()
     value_min = CFloat(None, allow_none=True)
@@ -651,6 +1235,8 @@ class ControlException(TraitError):
 
 
 class Control(Widget):
+    """Control abstract class."""
+
     _view_name = Unicode('LeafletControlView').tag(sync=True)
     _model_name = Unicode('LeafletControlModel').tag(sync=True)
     _view_module = Unicode('jupyter-leaflet').tag(sync=True)
@@ -940,7 +1526,8 @@ class SearchControl(Control):
     auto_collapse = Bool(False).tag(sync=True, o=True)
     animate_location = Bool(False).tag(sync=True, o=True)
 
-    marker = Instance(Marker).tag(sync=True, **widget_serialization)
+    marker = Instance(Marker, allow_none=True, default_value=None).tag(sync=True, **widget_serialization)
+    layer = Instance(LayerGroup, allow_none=True, default_value=None).tag(sync=True, **widget_serialization)
 
 
 class MapStyle(Style, Widget):
@@ -971,6 +1558,8 @@ class Map(DOMWidget, InteractMixin):
     zoom = CFloat(12).tag(sync=True, o=True)
     max_zoom = CFloat(18).tag(sync=True, o=True)
     min_zoom = CFloat(1).tag(sync=True, o=True)
+    zoom_delta = CFloat(1).tag(sync=True, o=True)
+    zoom_snap = CFloat(1).tag(sync=True, o=True)
     interpolation = Unicode('bilinear').tag(sync=True, o=True)
     crs = Dict(default_value=projections.EPSG3857).tag(sync=True)
 
@@ -1030,6 +1619,11 @@ class Map(DOMWidget, InteractMixin):
     east = Float(def_loc[1], read_only=True).tag(sync=True)
     west = Float(def_loc[1], read_only=True).tag(sync=True)
 
+    bottom = Float(0, read_only=True).tag(sync=True)
+    top = Float(9007199254740991, read_only=True).tag(sync=True)
+    right = Float(0, read_only=True).tag(sync=True)
+    left = Float(9007199254740991, read_only=True).tag(sync=True)
+
     layers = Tuple().tag(trait=Instance(Layer), sync=True, **widget_serialization)
 
     @default('layers')
@@ -1043,6 +1637,7 @@ class Map(DOMWidget, InteractMixin):
 
     bounds = Tuple(read_only=True)
     bounds_polygon = Tuple(read_only=True)
+    pixel_bounds = Tuple(read_only=True)
 
     @observe('south', 'north', 'east', 'west')
     def _observe_bounds(self, change):
@@ -1053,6 +1648,11 @@ class Map(DOMWidget, InteractMixin):
                                           (self.south, self.east),
                                           (self.south, self.west)))
 
+    @observe('bottom', 'top', 'right', 'left')
+    def _observe_pixel_bounds(self, change):
+        self.set_trait('pixel_bounds', ((self.left, self.top),
+                                        (self.right, self.bottom)))
+
     def __init__(self, **kwargs):
         self.zoom_control_instance = None
         self.attribution_control_instance = None
@@ -1095,7 +1695,7 @@ class Map(DOMWidget, InteractMixin):
         Makes sure only one instance of any given layer can exist in the
         layers list.
         '''
-        self._layer_ids = [l.model_id for l in proposal.value]
+        self._layer_ids = [layer.model_id for layer in proposal.value]
         if len(set(self._layer_ids)) != len(self._layer_ids):
             raise LayerException('duplicate layer detected, only use each layer once')
         return proposal.value
@@ -1105,19 +1705,19 @@ class Map(DOMWidget, InteractMixin):
             layer = basemap_to_tiles(layer)
         if layer.model_id in self._layer_ids:
             raise LayerException('layer already on map: %r' % layer)
-        self.layers = tuple([l for l in self.layers] + [layer])
+        self.layers = tuple([layer for layer in self.layers] + [layer])
 
-    def remove_layer(self, layer):
-        if layer.model_id not in self._layer_ids:
-            raise LayerException('layer not on map: %r' % layer)
-        self.layers = tuple([l for l in self.layers if l.model_id != layer.model_id])
+    def remove_layer(self, rm_layer):
+        if rm_layer.model_id not in self._layer_ids:
+            raise LayerException('layer not on map: %r' % rm_layer)
+        self.layers = tuple([layer for layer in self.layers if layer.model_id != rm_layer.model_id])
 
     def substitute_layer(self, old, new):
         if isinstance(new, dict):
             new = basemap_to_tiles(new)
         if old.model_id not in self._layer_ids:
             raise LayerException('Could not substitute layer: layer not on map.')
-        self.layers = tuple([new if l.model_id == old.model_id else l for l in self.layers])
+        self.layers = tuple([new if layer.model_id == old.model_id else layer for layer in self.layers])
 
     def clear_layers(self):
         self.layers = ()
