@@ -37,6 +37,7 @@ export class LeafletMapModel extends widgets.DOMWidgetModel {
       _model_module: 'jupyter-leaflet',
       _view_module: 'jupyter-leaflet',
       center: DEFAULT_LOCATION,
+      continuousWorld: false,
       zoom_start: 12,
       zoom: 12,
       max_zoom: 18,
@@ -86,25 +87,24 @@ export class LeafletMapModel extends widgets.DOMWidgetModel {
   }
 
   update_style() {
-    var new_style;
     if (!this.get('_dragging')) {
-      new_style = this.get('default_style');
+      var new_style = this.get('default_style');
     } else {
-      new_style = this.get('dragging_style');
+      var new_style = this.get('dragging_style');
     }
     this.set('style', new_style);
   }
 
   update_bounds() {
     return widgets.resolvePromisesDict(this.views).then(views => {
-      // default bounds if the projection is latlon
+    // default bounds if the projection is latlon
       var bounds = {
         north: -90,
         south: 90,
         east: -180,
         west: 180
       };
-      Object.keys(views).reduce(function (bnds, key) {
+      Object.keys(views).reduce(function(bnds, key) {
         var obj = views[key].obj;
         if (obj) {
           var view_bounds = obj.getBounds();
@@ -168,7 +168,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
   add_control_model(child_model) {
     return this.create_child_view(child_model, {
       map_view: this
-    }).then(view => {
+      }).then(view => {
       this.obj.addControl(view.obj);
 
       // Trigger the displayed event of the child view.
@@ -217,7 +217,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
   }
 
   create_obj() {
-    return this.layoutPromise.then(() => {
+    return this.layoutPromise.then(views => {
       var options = {
         ...this.get_options(),
         crs: proj.getProjection(this.model.get('crs')),
@@ -243,7 +243,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
       this.model.update_style();
     });
 
-    this.obj.on('movestart', () => {
+    this.obj.on('movestart', e => {
       this.model.set('_dragging', true);
       this.model.update_style();
     });
@@ -282,7 +282,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
     this.listenTo(
       this.model,
       'change:layers',
-      function () {
+      function() {
         this.layer_views.update(this.model.get('layers'));
       },
       this
@@ -290,7 +290,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
     this.listenTo(
       this.model,
       'change:controls',
-      function () {
+      function() {
         this.control_views.update(this.model.get('controls'));
       },
       this
@@ -298,7 +298,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
     this.listenTo(
       this.model,
       'change:zoom',
-      function () {
+      function() {
         if (!this.dirty) {
           this.dirty = true;
           // Using flyTo instead of setZoom to adjust for potential
@@ -322,7 +322,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
     this.listenTo(
       this.model,
       'change:center',
-      function () {
+      function() {
         if (!this.dirty) {
           this.dirty = true;
           this.obj.panTo(this.model.get('center'));
@@ -337,7 +337,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
     this.listenTo(
       this.model,
       'change:dragging_style',
-      function () {
+      function() {
         this.model.update_style();
       },
       this
@@ -345,7 +345,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
     this.listenTo(
       this.model,
       'change:default_style',
-      function () {
+      function() {
         this.model.update_style();
       },
       this
@@ -353,7 +353,7 @@ export class LeafletMapView extends utils.LeafletDOMWidgetView {
     this.listenTo(
       this.model,
       'change:fullscreen',
-      function () {
+      function() {
         var fullscreen = this.model.get('fullscreen');
         if (this.obj.isFullscreen() !== fullscreen) {
           this.obj.toggleFullscreen();
